using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Linq;
using SimpleJSON;
using GPUTools.Common.Scripts.Utils;
using UnityEngine.UI;

public class Persistence : MVRScript
{
    public Dictionary<string, ActionItem> actionsDictionary = new Dictionary<string, ActionItem>();

    public Context currentContext = new Context("", "", "");

    public ActionItem GetItemByName(string name)
    {
        if (actionsDictionary.Count > 0)
        {
            ActionItem actionItem;
            if (actionsDictionary.TryGetValue(name, out actionItem))
            {
                return actionItem;
            }

            SuperController.LogError("Action not found by name = " + name);
        }
        return null;
    }
    public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
    {
        JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
        if (includePhysical || forceStore)
        {
            needsStore = true;

            JSONArray actionsArray = new JSONArray();
            foreach (var kvp in actionsDictionary)
            {
                JSONClass actionJSON = new JSONClass();
                actionJSON["Name"] = kvp.Value.Name;
                actionJSON["Action"] = kvp.Value.Action;
                actionJSON["Group"] = kvp.Value.Group;
                actionJSON["When"] = kvp.Value.When;
                actionJSON["Effect"] = kvp.Value.Effect;
                actionJSON["Context"] = kvp.Value.Context;
                actionJSON["Location"] = kvp.Value.Location;
                actionJSON["Wearing"] = kvp.Value.Wearing;
                actionJSON["Deferred"].AsBool = kvp.Value.Deferred;
                actionJSON["Layer"] = kvp.Value.Layer;
                actionJSON["Trigger"] = kvp.Value.Trigger.GetJSON(base.subScenePrefix);

                JSONArray linkedActionsArray = new JSONArray();
                foreach (string linkedAction in kvp.Value.LinkedActions)
                {
                    linkedActionsArray.Add("", new JSONData(linkedAction));
                }
                actionJSON["LinkedActions"] = linkedActionsArray;

                actionsArray.Add(actionJSON);
            }

            JSONClass contextJSON = new JSONClass();
            contextJSON["FreeText"] = currentContext.FreeText;
            contextJSON["Wearing"] = currentContext.Wearing;
            contextJSON["Location"] = currentContext.Location;

            jc["Actions"] = actionsArray;
            jc["CurrentContext"] = contextJSON;
        }

        return jc;
    }

    public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
    {
        base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
        if (!base.physicalLocked && restorePhysical)
        {
            JSONArray actionsArray = jc["Actions"].AsArray;
            foreach (JSONClass actionJSON in actionsArray)
            {
                string actionName = actionJSON["Action"];
                string group = actionJSON["Group"];
                if (group == null)
                {
                    group = "";
                }
                string name = actionJSON["Name"];
                if (name == null)
                {
                    name = actionName;
                }
                string layer = actionJSON["Layer"];
                if (layer == null)
                {
                    layer = "";
                }
                string when = actionJSON["When"];
                string effect = actionJSON["Effect"];
                string context = actionJSON["Context"];
                string location = actionJSON["Location"];
                string wearing = actionJSON["Wearing"];
                bool deferred = actionJSON["Deferred"].AsBool;
                JSONClass triggerJSON = actionJSON["Trigger"].AsObject;
                EventTrigger trigger = new EventTrigger(this, "Trigger");
                trigger.RestoreFromJSON(triggerJSON);

                ActionItem actionItem = new ActionItem(name, actionName, group, when, effect, context, location, wearing, deferred, layer, trigger);

                JSONArray linkedActionsArray = actionJSON["LinkedActions"].AsArray;
                foreach (JSONNode linkedActionJSON in linkedActionsArray)
                {
                    string linkedAction = linkedActionJSON.Value;
                    actionItem.LinkedActions.Add(linkedAction);
                }

                actionsDictionary[name] = actionItem;
                JSONClass contextJSON = jc["CurrentContext"].AsObject;
                if (currentContext != null)
                {
                    currentContext = new Context(contextJSON["FreeText"], contextJSON["Wearing"], contextJSON["Location"]);
                }
            }
        }
    }

    private void OnDestroy()
    {
        Utils.OnDestroyUI();
    }

    public override bool ShouldIgnore() => true;
}
